#include <FK/FK.h>
#include "FKUT/FrameController.h"
#include "FKUT/Input.h"
#include <Fl/x.h>

#include "FKUT/Audio.h"

int main(int argc, char *argv[])
{
	Fl_Window		mainWindow(512, 512, "FK TEST");
	fk_Model		camera, blockModel, lightModel;
	fk_Light		light;
	fk_Block		block(50.0, 70.0, 40.0);
	fk_Scene		scene;
	fk_Window		window(0, 0, 512, 512);


	mainWindow.end();
	fk_InitMaterial();

	// Ɩ̐ݒ
	lightModel.setShape(&light);
	lightModel.setMaterial(White);
	lightModel.glMoveTo(0.0, 0.0, 0.0);
	lightModel.glFocus(-1.0, -1.0, -1.0);

	// ̂̐ݒ
	blockModel.setShape(&block);
	blockModel.setMaterial(Yellow);

	// effBXvCXgɓo^
	scene.entryCamera(&camera);
	scene.entryModel(&blockModel);
	scene.entryModel(&lightModel);

	// EBhEփfBXvCXgo^
	window.setScene(&scene);

	// _̈ʒuƎpݒ
	camera.glMoveTo(0.0, 0.0, 1000.0);
	camera.glFocus(0.0, 0.0, 0.0);
	camera.glUpvec(0.0, 1.0, 0.0);

	mainWindow.show();
	window.show();


	fkut_FrameController	fps;
	fkut_Input				input;
	fkut_InputInfo			joy;

	input.init(fl_display, fl_xid(&mainWindow));
	input.setKeyAssign(0, DIK_Z);
	input.setKeyAssign(1, DIK_S);
	input.setKeyAssign(2, DIK_X);
	input.setKeyAssign(3, DIK_D);
	input.setKeyAssign(4, DIK_C);
	input.setKeyAssign(5, DIK_F);
	input.setKeyAssign(6, DIK_V);
	input.setKeyAssign(7, DIK_G);
	input.setKeyAssign(8, DIK_B);
	input.setKeyAssign(9, DIK_SPACE);


	fk_AudioStream		bgm;
	fk_AudioWavBuffer	drum[10];
	bool				playFlag[10], prevKey[10];

	fk_AudioBase::setListenerModel(&camera);

	if(!bgm.open("BGM\\stage_01.ogg")) fl_alert("open() failed.");
	bgm.setLoopMode(true);

	drum[0].open("drum\\BDTIGHT2.wav");
	drum[1].open("drum\\CRSHCYM1.wav");
	drum[2].open("drum\\CRSHCYM1M.wav");
	drum[3].open("drum\\HH1FOOT.wav");
	drum[4].open("drum\\HH1OPEN1.wav");
	drum[5].open("drum\\HH1OPEN2.wav");
	drum[6].open("drum\\LOWTOM2.wav");
	drum[7].open("drum\\MIDTOM2.wav");
	drum[8].open("drum\\SDCRKRM.wav");
	drum[9].open("SE\\iPhone.wav");
	drum[9].setLoopMode(true);
	drum[9].setPosition(fk_Vector(0.0, 0.0, 0.0));
	drum[9].setReferenceDist(100.0);

	for(int i = 0; i < 10; i++) prevKey[i] = playFlag[i] = false;

	while(true) {
		if(mainWindow.visible() == 0) {
			if(Fl::wait() == 0) break;
			else continue;
		}

		if(window.drawWindow() == 0) break;
		if(Fl::check() == 0) break;
		if(window.winOpenStatus() == false) continue;

		// ̂ Y Sɉ]
		blockModel.glRotateWithVec(0.0, 0.0, 0.0, fk_Y, FK_PI/100.0);

		input.update();
		joy = input.getInputStatus();
		for(int i = 0; i < 10; i++) {
			if(i < 9) {
				if(joy.button[i] & !prevKey[i]) {
					playFlag[i] = true;
					drum[i].seek(0.0);
				}
				if(playFlag[i]) playFlag[i] = drum[i].play();
			} else {
				if(joy.button[i] & !prevKey[i]) {
					if(playFlag[i]) {
						playFlag[i] = false;
						bgm.pause();
					} else {
						playFlag[i] = bgm.play();
					}
				}
				if(playFlag[i]) playFlag[i] = bgm.play();
			}

			prevKey[i] = joy.button[i];
		}
		drum[9].play();

		if(window.getSpecialKeyStatus(FK_UP, false)) {
			camera.loTranslate(0.0, 0.0, -1.0);
		}
		if(window.getSpecialKeyStatus(FK_DOWN, false)) {
			camera.loTranslate(0.0, 0.0, 1.0);
		}
		if(window.getSpecialKeyStatus(FK_LEFT, false)) {
			camera.loFocus(-0.1, 0.0, -1.0);
		}
		if(window.getSpecialKeyStatus(FK_RIGHT, false)) {
			camera.loFocus(0.1, 0.0, -1.0);
		}
	}

	return 0;
}
